@@ -324,6 +324,11 @@ decset_decrst(struct terminal *term, unsigned param, bool enable)
         term->keypad_keys_mode = enable ? KEYPAD_APPLICATION : KEYPAD_NUMERICAL;
         break;
 
+    case 67:
+        if (enable)
+            LOG_WARN("unimplemented: DECBKM");
+        break;
+
     case 80:
         term->sixel.scrolling = !enable;
         break;
@@ -555,6 +560,7 @@ decrqm(const struct terminal *term, unsigned param)
     case 25: return decrpm(!term->hide_cursor);
     case 45: return decrpm(term->reverse_wrap);
     case 66: return decrpm(term->keypad_keys_mode == KEYPAD_APPLICATION);
+    case 67: return DECRPM_PERMANENTLY_RESET; /* https://vt100.net/docs/vt510-rm/DECBKM */
     case 80: return decrpm(!term->sixel.scrolling);
     case 1000: return decrpm(term->mouse_tracking == MOUSE_CLICK);
     case 1001: return DECRPM_PERMANENTLY_RESET;
@@ -600,6 +606,7 @@ xtsave(struct terminal *term, unsigned param)
     case 45: term->xtsave.reverse_wrap = term->reverse_wrap; break;
     case 47: term->xtsave.alt_screen = term->grid == &term->alt; break;
     case 66: term->xtsave.application_keypad_keys = term->keypad_keys_mode == KEYPAD_APPLICATION; break;
+    case 67: break;
     case 80: term->xtsave.sixel_display_mode = !term->sixel.scrolling; break;
     case 1000: term->xtsave.mouse_click = term->mouse_tracking == MOUSE_CLICK; break;
     case 1001: break;
@@ -642,6 +649,7 @@ xtrestore(struct terminal *term, unsigned param)
     case 45: enable = term->xtsave.reverse_wrap; break;
     case 47: enable = term->xtsave.alt_screen; break;
     case 66: enable = term->xtsave.application_keypad_keys; break;
+    case 67: return;
     case 80: enable = term->xtsave.sixel_display_mode; break;
     case 1000: enable = term->xtsave.mouse_click; break;
     case 1001: return;
@@ -673,6 +681,24 @@ xtrestore(struct terminal *term, unsigned param)
     decset_decrst(term, param, enable);
 }
 
+static bool
+params_to_rectangular_area(const struct terminal *term, int first_idx,
+                           int *top, int *left, int *bottom, int *right)
+{
+    int rel_top = vt_param_get(term, first_idx + 0, 1) - 1;
+    *left = min(vt_param_get(term, first_idx + 1, 1) - 1, term->cols - 1);
+    int rel_bottom = vt_param_get(term, first_idx + 2, term->rows) - 1;
+    *right = min(vt_param_get(term, first_idx + 3, term->cols) - 1, term->cols - 1);
+
+    if (rel_top > rel_bottom || *left > *right)
+        return false;
+
+    *top = term_row_rel_to_abs(term, rel_top);
+    *bottom = term_row_rel_to_abs(term, rel_bottom);
+
+    return true;
+}
+
 void
 csi_dispatch(struct terminal *term, uint8_t final)
 {
@@ -743,10 +769,10 @@ csi_dispatch(struct terminal *term, uint8_t final)
              * Note: tertiary DA responds with "FOOT".
              */
             if (term->conf->tweak.sixel) {
-                static const char reply[] = "\033[?62;4;22c";
+                static const char reply[] = "\033[?62;4;22;28c";
                 term_to_slave(term, reply, sizeof(reply) - 1);
             } else {
-                static const char reply[] = "\033[?62;22c";
+                static const char reply[] = "\033[?62;22;28c";
                 term_to_slave(term, reply, sizeof(reply) - 1);
             }
             break;
@@ -1081,44 +1107,29 @@ csi_dispatch(struct terminal *term, uint8_t final)
             break;
 
         case 'h':
-            /* Set mode */
-            switch (vt_param_get(term, 0, 0)) {
-            case 2:   /* Keyboard Action Mode - AM */
-                LOG_WARN("unimplemented: keyboard action mode (AM)");
-                break;
-
-            case 4:   /* Insert Mode - IRM */
-                term->insert_mode = true;
+        case 'l': {
+            /* Set/Reset Mode (SM/RM) */
+            int param = vt_param_get(term, 0, 0);
+            bool sm = final == 'h';
+            if (param == 4) {
+                /* Insertion Replacement Mode (IRM) */
+                term->insert_mode = sm;
                 term_update_ascii_printer(term);
                 break;
+            }
 
-            case 12:  /* Send/receive Mode - SRM */
-                LOG_WARN("unimplemented: send/receive mode (SRM)");
-                break;
-
-            case 20:  /* Automatic Newline Mode - LNM */
-                /* TODO: would be easy to implemented; when active
-                 * term_linefeed() would _also_ do a
-                 * term_carriage_return() */
-                LOG_WARN("unimplemented: automatic newline mode (LNM)");
-                break;
-            }
-            break;
-
-        case 'l':
-            /* Reset mode */
-            switch (vt_param_get(term, 0, 0)) {
-            case 4:   /* Insert Mode - IRM */
-                term->insert_mode = false;
-                term_update_ascii_printer(term);
-                break;
-
-            case 2:   /* Keyboard Action Mode - AM */
-            case 12:  /* Send/receive Mode - SRM */
-            case 20:  /* Automatic Newline Mode - LNM */
-                break;
+            /*
+             * ECMA-48 defines modes 1-22, all of which were optional
+             * (§7.1; "may have one state only") and are considered
+             * deprecated (§7.1) in the latest (5th) edition. xterm only
+             * documents modes 2, 4, 12 and 20, the last of which was
+             * outright removed (§8.3.106) in 5th edition ECMA-48.
+             */
+            if (sm) {
+                LOG_WARN("SM with unimplemented mode: %d", param);
             }
             break;
+        }
 
         case 'r': {
             int start = vt_param_get(term, 0, 1);
@@ -1216,9 +1227,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
                 if (width >= 0 && height >= 0) {
                     char reply[64];
                     size_t n = xsnprintf(
-                        reply, sizeof(reply), "\033[4;%d;%dt",
-                        (int)roundf(height / term->scale),
-                        (int)roundf((width / term->scale)));
+                        reply, sizeof(reply), "\033[4;%d;%dt", height, width);
                     term_to_slave(term, reply, n);
                 }
                 break;
@@ -1228,8 +1237,8 @@ csi_dispatch(struct terminal *term, uint8_t final)
                 tll_foreach(term->window->on_outputs, it) {
                     char reply[64];
                     size_t n = xsnprintf(reply, sizeof(reply), "\033[5;%d;%dt",
-                             it->item->dim.px_scaled.height,
-                             it->item->dim.px_scaled.width);
+                             it->item->dim.px_real.height,
+                             it->item->dim.px_real.width);
                     term_to_slave(term, reply, n);
                     break;
                 }
@@ -1242,8 +1251,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
                 char reply[64];
                 size_t n = xsnprintf(
                     reply, sizeof(reply), "\033[6;%d;%dt",
-                    (int)roundf(term->cell_height / term->scale),
-                    (int)roundf(term->cell_width / term->scale));
+                    term->cell_height, term->cell_width);
                 term_to_slave(term, reply, n);
                 break;
             }
@@ -1261,8 +1269,8 @@ csi_dispatch(struct terminal *term, uint8_t final)
                     char reply[64];
                     size_t n = xsnprintf(
                         reply, sizeof(reply), "\033[9;%d;%dt",
-                        (int)roundf(it->item->dim.px_real.height / term->cell_height / term->scale),
-                        (int)roundf(it->item->dim.px_real.width / term->cell_width / term->scale));
+                        it->item->dim.px_real.height / term->cell_height,
+                        it->item->dim.px_real.width / term->cell_width);
                     term_to_slave(term, reply, n);
                     break;
                 }
@@ -1444,6 +1452,23 @@ csi_dispatch(struct terminal *term, uint8_t final)
             break;
         }
 
+        case 'p': {
+            /*
+             * Request status of ECMA-48/"ANSI" private mode (DECRQM
+             * for SM/RM modes; see private="?$" case further below for
+             * DECSET/DECRST modes)
+             */
+            unsigned param = vt_param_get(term, 0, 0);
+            unsigned status = DECRPM_NOT_RECOGNIZED;
+            if (param == 4) {
+                status = decrpm(term->insert_mode);
+            }
+            char reply[32];
+            size_t n = xsnprintf(reply, sizeof(reply), "\033[%u;%u$y", param, status);
+            term_to_slave(term, reply, n);
+            break;
+        }
+
         case 'u': {
             enum kitty_kbd_flags flags =
                 term->grid->kitty_kbd.flags[term->grid->kitty_kbd.idx];
@@ -1551,8 +1576,8 @@ csi_dispatch(struct terminal *term, uint8_t final)
                 break;
 
             case 4:  /* modifyOtherKeys */
-                /* We don’t support fully disabling modifyOtherKeys,
-                 * but simply revert back to mode ‘1’ */
+                /* We don't support fully disabling modifyOtherKeys,
+                 * but simply revert back to mode '1' */
                 term->modify_other_keys_2 = false;
                 LOG_DBG("modifyOtherKeys=1");
                 break;
@@ -1630,7 +1655,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
             break;
         }
         }
-        break; /* private[0] == ‘<’ */
+        break; /* private[0] == '<' */
     }
 
     case ' ': {
@@ -1750,6 +1775,214 @@ csi_dispatch(struct terminal *term, uint8_t final)
         break; /* private[0] == '=' */
     }
 
+    case '$': {
+        switch (final) {
+        case 'r': {  /* DECCARA */
+            int top, left, bottom, right;
+            if (!params_to_rectangular_area(
+                    term, 0, &top, &left, &bottom, &right))
+            {
+                break;
+            }
+
+            for (int r = top; r <= bottom; r++) {
+                struct row *row = grid_row(term->grid, r);
+                row->dirty = true;
+
+                for (int c = left; c <= right; c++) {
+                    struct attributes *a = &row->cells[c].attrs;
+                    a->clean = 0;
+
+                    for (size_t i = 4; i < term->vt.params.idx; i++) {
+                        const int param = term->vt.params.v[i].value;
+
+                        /* DECCARA only supports a sub-set of SGR parameters */
+                        switch (param) {
+                        case 0:
+                            a->bold = false;
+                            a->underline = false;
+                            a->blink = false;
+                            a->reverse = false;
+                            break;
+
+                        case 1: a->bold = true; break;
+                        case 4: a->underline = true; break;
+                        case 5: a->blink = true; break;
+                        case 7: a->reverse = true; break;
+
+                        case 22: a->bold = false; break;
+                        case 24: a->underline = false; break;
+                        case 25: a->blink = false; break;
+                        case 27: a->reverse = false; break;
+                        }
+                    }
+                }
+            }
+            break;
+        }
+
+        case 't': {  /* DECRARA */
+            int top, left, bottom, right;
+            if (!params_to_rectangular_area(
+                    term, 0, &top, &left, &bottom, &right))
+            {
+                break;
+            }
+
+            for (int r = top; r <= bottom; r++) {
+                struct row *row = grid_row(term->grid, r);
+                row->dirty = true;
+
+                for (int c = left; c <= right; c++) {
+                    struct attributes *a = &row->cells[c].attrs;
+                    a->clean = 0;
+
+                    for (size_t i = 4; i < term->vt.params.idx; i++) {
+                        const int param = term->vt.params.v[i].value;
+
+                        /* DECRARA only supports a sub-set of SGR parameters */
+                        switch (param) {
+                        case 0:
+                            a->bold = !a->bold;
+                            a->underline = !a->underline;
+                            a->blink = !a->blink;
+                            a->reverse = !a->reverse;
+                            break;
+
+                        case 1: a->bold = !a->bold; break;
+                        case 4: a->underline = !a->underline; break;
+                        case 5: a->blink = !a->blink; break;
+                        case 7: a->reverse = !a->reverse; break;
+                        }
+                    }
+                }
+            }
+            break;
+        }
+
+        case 'v': {  /* DECCRA */
+            int src_top, src_left, src_bottom, src_right;
+            if (!params_to_rectangular_area(
+                    term, 0, &src_top, &src_left, &src_bottom, &src_right))
+            {
+                break;
+            }
+
+            int src_page = vt_param_get(term, 4, 1);
+
+            int dst_rel_top = vt_param_get(term, 5, 1) - 1;
+            int dst_left = vt_param_get(term, 6, 1) - 1;
+            int dst_page = vt_param_get(term, 7, 1);
+
+            if (unlikely(src_page != 1 || dst_page != 1)) {
+                /* We don’t support “pages” */
+                break;
+            }
+
+            int dst_rel_bottom = dst_rel_top + (src_bottom - src_top);
+            int dst_right = min(dst_left + (src_right - src_left), term->cols - 1);
+
+            int dst_top = term_row_rel_to_abs(term, dst_rel_top);
+            int dst_bottom = term_row_rel_to_abs(term, dst_rel_bottom);
+
+            /* Target area outside the screen is clipped */
+            const size_t row_count = min(src_bottom - src_top,
+                                         dst_bottom - dst_top) + 1;
+            const size_t cell_count = min(src_right - src_left,
+                                          dst_right - dst_left) + 1;
+
+            sixel_overwrite_by_rectangle(
+                term, dst_top, dst_left, row_count, cell_count);
+
+            /*
+             * Copy source area
+             *
+             * Note: since source and destination may overlap, we need
+             * to copy out the entire source region first, and _then_
+             * write the destination. I.e. this is similar to how
+             * memmove() behaves, but adapted to our row/cell
+             * structure.
+             */
+            struct cell **copy = xmalloc(row_count * sizeof(copy[0]));
+            for (int r = 0; r < row_count; r++) {
+                copy[r] = xmalloc(cell_count * sizeof(copy[r][0]));
+
+                const struct row *row = grid_row(term->grid, src_top + r);
+                const struct cell *cell = &row->cells[src_left];
+                memcpy(copy[r], cell, cell_count * sizeof(copy[r][0]));
+            }
+
+            /* Paste into destination area */
+            for (int r = 0; r < row_count; r++) {
+                struct row *row = grid_row(term->grid, dst_top + r);
+                row->dirty = true;
+
+                struct cell *cell = &row->cells[dst_left];
+                memcpy(cell, copy[r], cell_count * sizeof(copy[r][0]));
+                free(copy[r]);
+
+                for (;cell < &row->cells[dst_left + cell_count]; cell++)
+                    cell->attrs.clean = 0;
+
+                if (unlikely(row->extra != NULL)) {
+                    /* TODO: technically, we should copy the source URIs... */
+                    grid_row_uri_range_erase(row, dst_left, dst_right);
+                }
+            }
+            free(copy);
+            break;
+        }
+
+        case 'x': {  /* DECFRA */
+            const uint8_t c = vt_param_get(term, 0, 0);
+
+            if (unlikely(!((c >= 32 && c < 126) || c >= 160)))
+                break;
+
+            int top, left, bottom, right;
+            if (!params_to_rectangular_area(
+                    term, 1, &top, &left, &bottom, &right))
+            {
+                break;
+            }
+
+            /* Erase the entire region at once (MUCH cheaper than
+             * doing it row by row, or even character by
+             * character). */
+            sixel_overwrite_by_rectangle(
+                term, top, left, bottom - top + 1, right - left + 1);
+
+            for (int r = top; r <= bottom; r++)
+                term_fill(term, r, left, c, right - left + 1, true);
+
+            break;
+        }
+
+        case 'z': {  /* DECERA */
+            int top, left, bottom, right;
+            if (!params_to_rectangular_area(
+                    term, 0, &top, &left, &bottom, &right))
+            {
+                break;
+            }
+
+            /*
+             * Note: term_erase() _also_ erases sixels, but since
+             * we’re forced to erase one row at a time, erasing the
+             * entire sixel here is more efficient.
+             */
+            sixel_overwrite_by_rectangle(
+                term, top, left, bottom - top + 1, right - left + 1);
+
+            for (int r = top; r <= bottom; r++)
+                term_erase(term, r, left, r, right);
+            break;
+        }
+        }
+
+        break; /* private[0] == ‘$’ */
+    }
+
     case 0x243f:  /* ?$ */
         switch (final) {
         case 'p': {
@@ -1777,7 +2010,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
             break;
         }
 
-        break; /* private[0] == ‘?’ && private[1] == ‘$’ */
+        break; /* private[0] == '?' && private[1] == '$' */
 
     default:
         UNHANDLED();
