@@ -18,6 +18,7 @@
 #include "debug.h"
 #include "grid.h"
 #include "osc.h"
+#include "sixel.h"
 #include "util.h"
 #include "xmalloc.h"
 
@@ -137,12 +138,12 @@ action_execute(struct terminal *term, uint8_t c)
         /* backspace */
 #if 0
         /*
-         * This is the “correct” BS behavior. However, it doesn’t play
+         * This is the "correct" BS behavior. However, it doesn't play
          * nicely with bw/auto_left_margin, hence the alternative
          * implementation below.
          *
-         * Note that it breaks vttest “1. Test of cursor movements ->
-         * Test of autowrap”
+         * Note that it breaks vttest "1. Test of cursor movements ->
+         * Test of autowrap"
          */
         term_cursor_left(term, 1);
 #else
@@ -154,7 +155,7 @@ action_execute(struct terminal *term, uint8_t c)
                 likely(term->reverse_wrap && term->auto_margin))
             {
                 if (term->grid->cursor.point.row <= term->scroll_region.start) {
-                    /* Don’t wrap past, or inside, the scrolling region(?) */
+                    /* Don't wrap past, or inside, the scrolling region(?) */
                 } else
                     term_cursor_to(
                         term,
@@ -398,7 +399,7 @@ action_collect(struct terminal *term, uint8_t c)
      * more.
      *
      * As such, we optimize *reading* the private(s), and *resetting*
-     * them (in action_clear()). Writing is ok if it’s a bit slow.
+     * them (in action_clear()). Writing is ok if it's a bit slow.
      */
 
     if ((term->vt.private & 0xff) == 0)
@@ -560,15 +561,16 @@ action_esc_dispatch(struct terminal *term, uint8_t final)
 
     case '#':
         switch (final) {
-        case '8':
-            for (int r = 0; r < term->rows; r++) {
-                struct row *row = grid_row(term->grid, r);
-                for (int c = 0; c < term->cols; c++) {
-                    row->cells[c].wc = U'E';
-                    row->cells[c].attrs = (struct attributes){0};
-                }
-                row->dirty = true;
-            }
+        case '8':  /* DECALN */
+            sixel_overwrite_by_rectangle(term, 0, 0, term->rows, term->cols);
+
+            term->scroll_region.start = 0;
+            term->scroll_region.end = term->rows;
+
+            for (int r = 0; r < term->rows; r++)
+                term_fill(term, r, 0, 'E', term->cols, false);
+
+            term_cursor_home(term);
             break;
         }
         break;  /* private[0] == '#' */
@@ -783,7 +785,7 @@ action_utf8_print(struct terminal *term, char32_t wc)
 
                 /*
                  * We may have a key collisison, so need to check that
-                 * it’s a true match. If not, bump the key and try
+                 * it's a true match. If not, bump the key and try
                  * again.
                  */
 
@@ -850,8 +852,20 @@ action_utf8_print(struct terminal *term, char32_t wc)
                 break;
 
             case GRAPHEME_WIDTH_DOUBLE:
-                if (unlikely(wc == 0xfe0f))
-                    width = 2;
+#if defined(FOOT_GRAPHEME_CLUSTERING)
+                if (unlikely(grapheme_clustering &&
+                             wc == 0xfe0f &&
+                             new_cc->count == 2))
+                {
+                    /* Only emojis should be affected by VS16 */
+                    const utf8proc_property_t *props =
+                        utf8proc_get_property(new_cc->chars[0]);
+
+                    if (props->boundclass == UTF8PROC_BOUNDCLASS_EXTENDED_PICTOGRAPHIC)
+                        width = 2;
+                }
+#endif
+
                 new_cc->width = min(grapheme_width + width, 2);
                 break;
 
@@ -920,8 +934,8 @@ action_utf8_33(struct terminal *term, uint8_t c)
         return;
     }
 
-    /* Note: the E0 range contains overlong encodings. We don’t try to
-       detect, as they’ll still decode to valid UTF-32. */
+    /* Note: the E0 range contains overlong encodings. We don't try to
+       detect, as they'll still decode to valid UTF-32. */
 
     action_utf8_print(term, term->vt.utf8);
 }
@@ -960,8 +974,8 @@ action_utf8_44(struct terminal *term, uint8_t c)
         return;
     }
 
-    /* Note: the F0 range contains overlong encodings. We don’t try to
-       detect, as they’ll still decode to valid UTF-32. */
+    /* Note: the F0 range contains overlong encodings. We don't try to
+       detect, as they'll still decode to valid UTF-32. */
 
     action_utf8_print(term, term->vt.utf8);
 }
