@@ -1,8 +1,6 @@
 #include <errno.h>
-#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include "xmalloc.h"
 #include "debug.h"
 
